# -*- coding: utf-8 -*-

__all__ = ['battery', 'decrypter', 'sensors', 'emotiv', 'packet', 'util', 'reader']
"""
emokit - emotiv epoc reverse engineering
~~~~~~~~~~~~~~~

:copyright: Copyright (c) 2010-2012, Cody Brocious, Kyle Machulis/Nonpolynomial Labs, Copyright 2013 Bryan Bishop
    Copyright 2012-2016 Bill Schumacher/Cerebral Power
:license: Public Domain, See LICENSE
"""

__title__ = "emokit"
__version__ = "0.0.7"
__build__ = 0x000007
__author__ = "Cody Brocious / Kyle Machulis"
__license__ = "Public Domain, See LICENSE"
__copyright__ = "Copyright (c) 2010-2012, Cody Brocious, Kyle Machulis/Nonpolynomial Labs," \
                " Copyright 2013 Bryan Bishop, Copyright 2012-2016 Bill Schumacher/Cerebral Power"
